/***********************************************************
 * $Id: DataCodeable.java 223 2010-04-09 20:28:48Z wglas $
 * 
 * optional of the clazzes.org project
 * 
 * http://www.clazzes.org
 *
 * Created: 09.04.2010, wglas
 *
 ***********************************************************/

package org.clazzes.optional.io;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 * <p>This interface is implemented by classes which may be serialized to
 * a {@link DataOutput} instance and deserialized from a {@link DataInput}
 * instance.</p>
 * 
 * <p>By convention, each implementation should provide for a static
 * <code>readDataInstance(DataInput)</code>
 * method, which constructs a new instance from the stream:</p>
 * <pre>
 * import java.io.DataInput;
 * import java.io.DataOutput;
 * class MyDataCodeable {
 *    public static MyDataCodeable readDataInstance(DataInput in) throws IOException {
 *      // read fields, construct new instance...
 *    }
 *    
 *    public void writeData(DataOutput out) throws IOException {
 *      // write fields...
 *    }
 * }
 * </pre>
 * 
 * @see DataCodingHelper
 */
public interface DataCodeable {

    /**
     * Write fields of this instance to the given {@link DataOutput} instance.
     * @param out The output to write to.
     * @throws IOException Upon write errors.
     */
    public void writeData(DataOutput out) throws IOException;
}
