/***********************************************************
 * $Id: AEADBlockCipher.java 181 2010-02-06 21:48:24Z wglas $
 * 
 * optional of the clazzes.org project
 * 
 * http://www.clazzes.org
 *
 * Created: 28.01.2010, wglas
 *
 ***********************************************************/

package org.clazzes.optional.sec;

/**
 * A block cipher mode that includes authenticated encryption with a streaming mode and optional associated data.
 * Copied from bouncycastle.
 */
public interface AEADBlockCipher extends BlockCipher {

    /**
     * Initialize the block cipher.
     * @param aad The additional authenticated data or
     *           <code>null</code>, if no additional authenticated data is needed.
     * @param macBits The length of the calculated myac in bits.
     * @param iv The initialization (nonce) vector or
     *           <code>null</code>, if no initialization vector is needed.
     * @param forEncryption Whether we want to encrypt. <code>false</code>
     *             means to set up this cipher for decryption.
     */
    public void init(byte[] aad, int macBits, byte[] iv, boolean forEncryption);

    /**
     * Proceed the next block of <code>in_len</code> bytes from the input.
     * 
     * This complements the functionality of {@link #processBlock(byte[], int, byte[], int)},
     * because AEAD ciphers may process arbitrary data length.
     * 
     * @param in The buffer where the input is stored.
     *           {@link #getBlockSize()} bytes starting with <code>in_off</code>
     *           will be consumed.
     * @param in_off The offset at which the input data starts.
     * @param in_len The length of the input data to process.
     * @param out The buffer where the output is stored.
     *           {@link #getBlockSize()} bytes starting with <code>out_off</code>
     *           will be written.
     * @param out_off The offset at which the output will be stored.
     * @return The block size aka the number of bytes written. 
     */
    public int processBytes(byte[] in, int in_off, int in_len, byte[] out, int out_off);

    /**
     * Finish the operation either appending or verifying the MAC at the end of the data.
     *
     * @param out space for any resulting output data.
     * @param outOff offset into out to start copying the data at.
     * @return number of bytes written into out.
     * @throws InvalidCipherTextException if the MAC fails to match.
     */
    public int doFinal(byte[] out, int outOff) throws InvalidCipherTextException;

    /**
     * Return the value of the MAC associated with the last stream processed.
     *
     * @return MAC for plaintext data.
     */
    public byte[] getMac();
    
    /**
     * 
     * @param len The length of the input data.
     * @return The length of the generated output data.
     */
    public int getOutputSize(int len);
}
