/***********************************************************
 * $Id: BlockCipher.java 187 2010-02-20 19:04:42Z wglas $
 * 
 * optional of the clazzes.org project
 * 
 * http://www.clazzes.org
 *
 * Created: 25.01.2010, wglas
 *
 ***********************************************************/

package org.clazzes.optional.sec;

/**
 * An interface, which encapsulates symmetric ciphers like 3-DES or AES.
 */
public interface BlockCipher {

    /**
     * Return the name of the algorithm the cipher implements.
     *
     * @return the name of the algorithm the cipher implements.
     */
   public String getAlgorithmName();

    /**
     * @return The block size used for the {@link #processBlock(byte[], int, byte[], int)} operation.
     */
    public int getBlockSize();
    
    /**
     * Initialize the block cipher.
     * @param key The key for basic ciphers or initialization vector for cipher modes
     *           or <code>null</code>, if no key or initialization vector is needed.
     * @param forEncryption Whether we want to encrypt. <code>false</code>
     *             means to set up this cipher for decryption.
     */
    public void init(byte[] key, boolean forEncryption);
    
    /**
     * Proceed the next block of {@link #getBlockSize()} bytes from the input.
     * 
     * @param in The buffer where the input is stored.
     *           {@link #getBlockSize()} bytes starting with <code>in_off</code>
     *           will be consumed.
     * @param in_off The offset at which the input data starts.
     * @param out The buffer where the output is stored.
     *           {@link #getBlockSize()} bytes starting with <code>out_off</code>
     *           will be written.
     * @param out_off The offset at which the output will be stored.
     * @return The block size aka the number of bytes written. 
     */
    public int processBlock(byte[] in, int in_off, byte[] out, int out_off);

    /**
     * Reset this cipher to state of the last call to {@link #init(byte[], byte[], boolean)}.
     */
    public void reset();

    /**
     * return the underlying block cipher that we are wrapping or
     * <code>null</code>, if we do wrap another cipher.
     *
     * @return the underlying block cipher that we are wrapping.
     */
    public BlockCipher getUnderlyingCipher();
    
}
