/***********************************************************
 * $Id: Digest.java 177 2010-02-04 20:48:29Z wglas $
 * 
 * optional of the clazzes.org project
 * 
 * http://www.clazzes.org
 *
 * Created: 25.01.2010, wglas
 *
 ***********************************************************/

package org.clazzes.optional.sec;

/**
 * An interface, which encapsulates secure hash function like SHA-1 or SHA-256.
 */
public interface Digest {
    
    /**
     * @return The name of the digest algorithm.
     */
    public String getAlgorithmName();
    
    /**
     * @return The size of computed hashes.
     */
    public int getDigestSize();
    
    /**
     * @return A new digest in identical state. This is used, when you
     *         want to calculate an intermediate digest value and continue
     *         afterward with adding more content.
     */
    public Digest cloneState();
    
    /**
     * Return the size in bytes of the internal buffer the digest applies it's compression
     * function to.
     * 
     * @return byte length of the digests internal buffer.
     */
    public int getByteLength();

    /**
     * Reset the hash in order to reuse this instance.
     */
    public void reset();
    
    /**
     * update the message digest with a single byte.
     *
     * @param in the input byte to be entered.
     */
    public void update(byte in);

    /**
     * Add more data to the hash function.
     * 
     * @param data The buffer where the data to add is stored.
     *          <code>length</code> bytes starting with <code>off</code>
     *           will be written.
     * @param off The offset at which the data is stored.
     * @param length The number of bytes to add. 
     */
    public void update(byte[] data, int off, int length);
    
    /**
     * Retrieve the hash of all data added so far with
     * {@link #update(byte[], int, int)}. 
     * @param hash The buffer, where the computed hash will be stored.
     *        {@link #getDigestSize()} bytes starting at <code>off</code>
     *        will be written to <code>hash</code>.
     * @param off The offset at which the hash is stored.
     * @return The number of bytes written, which is equal to
     *         {@link #getDigestSize()}.
     */
    public int doFinal(byte[] hash, int off);
}
