/***********************************************************
 * $Id: Mac.java 172 2010-01-29 16:44:32Z wglas $
 * 
 * optional of the clazzes.org project
 * 
 * http://www.clazzes.org
 *
 * Created: 27.01.2010, wglas
 *
 ***********************************************************/

package org.clazzes.optional.sec;

/**
 * A message authentication code.
 *
 */
public interface Mac {
    
    /**
     * Initialise the MAC.
     *
     * @param key the key required by the MAC.
     */
    public void init(byte []key);

    /**
     * Return the name of the algorithm the MAC implements.
     *
     * @return the name of the algorithm the MAC implements.
     */
    public String getAlgorithmName();

    /**
     * Return the block size for this MAC (in bytes).
     *
     * @return the block size for this MAC in bytes.
     */
    public int getMacSize();

    /**
     * add a single byte to the mac for processing.
     *
     * @param in the byte to be processed.
     */
    public void update(byte in);

    /**
     * @param in the array containing the input.
     * @param inOff the index in the array the data begins at.
     * @param len the length of the input starting at inOff.
     */
    public void update(byte[] in, int inOff, int len);

    /**
     * Compute the final stage of the MAC writing the output to the out
     * parameter.
     * <p>
     * doFinal leaves the MAC in the same state it was after the last init.
     *
     * @param out the array the MAC is to be output to.
     * @param outOff the offset into the out buffer the output is to start at.
     */
    public int doFinal(byte[] out, int outOff);

    /**
     * Reset the MAC. At the end of resetting the MAC should be in the
     * in the same state it was after the last init (if there was one).
     */
    public void reset();

}
