/***********************************************************
 * $Id: Pack.java 175 2010-02-01 17:26:35Z wglas $
 * 
 * optional of the clazzes.org project
 * 
 * http://www.clazzes.org
 *
 * Created: 28.01.2010, wglas
 *
 ***********************************************************/

package org.clazzes.optional.sec;

/**
 * Helper methods for packing/unpacking data.
 */
abstract public class Pack {

    public static byte[] getSubArray(byte [] data, int off, int len) {
        
        byte[] ret = new byte[len];
        System.arraycopy(data,off,ret,0,len);
        return ret;
    }
    
    /**
     * A constant time equals comparison - does not terminate early if
     * test will fail.
     *
     * @param a first array
     * @param b second array
     * @return true if arrays equal, false otherwise.
     */
    public static boolean constantTimeAreEqual(
        byte[]  a,
        byte[]  b)
    {
        if (a == b)
        {
            return true;
        }
    
        if (a == null || b == null)
        {
            return false;
        }
    
        if (a.length != b.length)
        {
            return false;
        }
    
        int nonEqual = 0;
    
        for (int i = 0; i != a.length; i++)
        {
            nonEqual |= (a[i] ^ b[i]);
        }
    
        return nonEqual == 0;
    }

    public static int bigEndianToInt(byte[] bs, int off)
    {
        int n = bs[  off] << 24;
        n |= (bs[++off] & 0xff) << 16;
        n |= (bs[++off] & 0xff) << 8;
        n |= (bs[++off] & 0xff);
        return n;
    }

    public static void intToBigEndian(int n, byte[] bs, int off)
    {
        bs[  off] = (byte)(n >>> 24);
        bs[++off] = (byte)(n >>> 16);
        bs[++off] = (byte)(n >>>  8);
        bs[++off] = (byte)(n       );
    }

    public static byte[] clone(byte[] data)
    {
        if (data == null)
        {
            return null;
        }
        byte[] copy = new byte[data.length];
    
        System.arraycopy(data, 0, copy, 0, data.length);
    
        return copy;
    }

}
