/***********************************************************
 * $Id: MGF.java 216 2010-03-25 22:25:41Z wglas $
 * 
 * optional of the clazzes.org project
 * 
 * http://www.clazzes.org
 *
 * Created: 23.03.2010, wglas
 *
 ***********************************************************/

package org.clazzes.optional.sec.rsa;

import org.clazzes.optional.sec.Digest;

/**
 * A mask generation function for the OAEP algorithm as defined in 
 * <a href="http://tools.ietf.org/html/rfc3447#appendix-B.2">Appendix B.2 of RFC 3447</a>.
 */
public interface MGF {

    /**
     * Generate <code>maskLen</code> mask bytes from a seed.
     * @param mgfSeed The seed value.
     * @param seedOff The offset of the seed in <code>mgfSeed</code>.
     * @param seedLen The number of seed bytes <code>mgfSeed</code> to use.
     * @param ret The byte array where the result is beings stored.
     * @param off The offset in <code>ret</code> where to store the result.
     * @param maskLen The number of octets to generate.
     * @return An array of pseudo-random, <code>maskLen</code> bytes.
     */
    byte [] generateMask(byte[] mgfSeed, int seedOff, int seedLen, byte[] ret, int off, int maskLen);

    /**
     * @return The message digest, which is used by this mask generator.
     */
    Digest getUnderlyingDigest();
}
