/***********************************************************
 * $Id: RSAPubKey.java 222 2010-04-05 19:51:13Z wglas $
 * 
 * optional of the clazzes.org project
 * 
 * http://www.clazzes.org
 *
 * Created: 25.03.2010, wglas
 *
 ***********************************************************/

package org.clazzes.optional.sec.rsa;

import org.clazzes.optional.io.Serializable;
import org.clazzes.optional.lang.BigInt;

/**
 * A RSA public key with a public exponent and a modulus.
 */
public class RSAPubKey implements Serializable {
    
    private static final long serialVersionUID = 7133263230069828013L;

    private final BigInt modulus;
    private final BigInt exponent;
        
    /**
     * Construct a new RSA public key by the given modulus and public exponent.
     * @param modulus The RSA modulus.
     * @param exponent The public exponent.
     */
    public RSAPubKey (BigInt modulus, BigInt exponent) {
        this.modulus = modulus;
        this.exponent = exponent;
    }

    /**
     * @return The modulus of the RAS public key.
     */
    public BigInt getModulus() {
        return this.modulus;
    }

    /**
     * @return The public exponent of the RSA key.
     */
    public BigInt getExponent() {
        return this.exponent;
    }

    /**
     * Calculates the plain encrpyt operation
     * <code>x^pubExp</code>. This operation may additionally
     * be used on a padded message digest to verify signatures.
     * 
     * @param plain The plain text, which is usually subject to
     *         an appropriate padding scheme.
     * @return The encrypted cipher text.
     */
    public BigInt encryptPlain(BigInt plain) {
        return plain.powMod(this.exponent,this.modulus);
    }
}
